@@ -243,109 +243,153 @@ class ProblemDefinition:
                              "Should be (num_ODE + num_parameters - num_left_boundary_conditions, num_ODE, num_parameters)")
 
         # test derivatives
-        step = 1e-10
-        places = 5
+        step = 1e-8
+        places = 4
+
+        # chose the point near the middle of the test_solution to check the derivatives
+        middlePoint = numpy.round(test_solution.mesh.size * .61, 0)
+
+
         if self.has_function_derivative:
             if self.num_parameters > 0:
 
-                func_derivative_calc, func_param_derivative_calc = self.function_derivativep(test_solution.mesh[1], test_solution.solution[:, 1], test_solution.parameters)
+                func_derivative_calc, func_param_derivative_calc = self.function_derivativep(test_solution.mesh[middlePoint], test_solution.solution[:, middlePoint], test_solution.parameters)
                 func_derivative_num = numpy.zeros((self.num_ODE,self.num_ODE))
                 func_param_derivative_num = numpy.zeros( (self.num_ODE, self.num_parameters))
 
+
+
                 for i in range(self.num_ODE):
                     delta = numpy.zeros(self.num_ODE)
                     delta[i] += step
 
-                    point1 = self.functionp(test_solution.mesh[1], test_solution.solution[:, 1], test_solution.parameters)
-                    point2 = self.functionp(test_solution.mesh[1], test_solution.solution[:, 1] + delta, test_solution.parameters)
+                    point1 = self.functionp(test_solution.mesh[middlePoint], test_solution.solution[:, middlePoint], test_solution.parameters)
+                    point2 = self.functionp(test_solution.mesh[middlePoint], test_solution.solution[:, middlePoint] + delta, test_solution.parameters)
 
-                    func_derivative_num[:, i] = (point1 - point2)/step
+                    func_derivative_num[:, i] = (point2 - point1)/step
 
                 for i in range(self.num_parameters):
                     delta = numpy.zeros(self.num_parameters)
                     delta[i] += step
 
-                    point1 = self.functionp(test_solution.mesh[1], test_solution.solution[:, 1], test_solution.parameters)
-                    point2 = self.functionp(test_solution.mesh[1], test_solution.solution[:, 1], test_solution.parameters + delta)
+                    point1 = self.functionp(test_solution.mesh[middlePoint], test_solution.solution[:, middlePoint], test_solution.parameters)
+                    point2 = self.functionp(test_solution.mesh[middlePoint], test_solution.solution[:, middlePoint], test_solution.parameters + delta)
 
                     func_param_derivative_num[:, i] = (point2 - point1)/step
 
                 # now compare the actual derivatives with the calculated ones
 
-                if not (numpy.round(func_derivative_calc - func_derivative_num, places) == 1).all():
-                    raise ValueError("analytical derivative matrix does not match numerical derivative matrix. Difference is \n" + str(func_derivative_calc - func_derivative_num))
+                difference = func_derivative_calc - func_derivative_num
+                if not (numpy.round(difference, places) == 0).all():
+                    raise ValueError("analytical derivative matrix does not match numerical derivative matrix.\n Analytical is:\n" + str(func_derivative_num)
+                                     + "\nNumerical is:\n" + str(func_derivative_num))
 
-                if not (numpy.round(func_param_derivative_calc - func_param_derivative_num, places) == 0).all():
-                    raise ValueError("analytical derivative (with respect to parameters) matrix does not match numerical derivative matrix. Difference is \n" + str(func_param_derivative_calc - func_param_derivative_num))
+                parameterDifference = func_param_derivative_calc - func_param_derivative_num
+                if not (numpy.round(parameterDifference, places) == 0).all():
+                    raise ValueError("analytical derivative (with respect to parameters) matrix does not match numerical derivative matrix.\n Analytical is:\n" + str(func_param_derivative_num)
+                                     + "\nNumerical is:\n" + str(func_param_derivative_num))
             else:
-                func_derivative_calc = self.function_derivative(test_solution.mesh[1], test_solution.solution[:, 1])
+
+                func_derivative_calc = self.function_derivative(test_solution.mesh[middlePoint], test_solution.solution[:, middlePoint])
                 func_derivative_num = numpy.zeros((self.num_ODE,self.num_ODE))
 
                 for i in range(self.num_ODE):
                     delta = numpy.zeros(self.num_ODE)
                     delta[i] += step
 
-                    point1 = self.function(test_solution.mesh[1], test_solution.solution[:, 1])
-                    point2 = self.function(test_solution.mesh[1], test_solution.solution[:, 1] + delta)
+                    point1 = self.function(test_solution.mesh[middlePoint], test_solution.solution[:, middlePoint])
+                    point2 = self.function(test_solution.mesh[middlePoint], test_solution.solution[:, middlePoint] + delta)
 
                     func_derivative_num[:, i] = (point2 - point1)/step
 
-
                 # now compare the actual derivatives with the calculated ones
-                if not (numpy.round(func_derivative_calc - func_derivative_num, places) == 0).all():
-                    raise ValueError("analytical derivative matrix does not match numerical derivative matrix. Difference is \n" + str(func_derivative_calc - func_derivative_num))
-
+                difference = func_derivative_calc - func_derivative_num
+                if not (numpy.round(difference, places) == 0).all():
+                    raise ValueError("analytical derivative matrix does not match numerical derivative matrix.\n Analytical is:\n" + str(func_derivative_num)
+                                     + "\nNumerical is:\n" + str(func_derivative_num))
 
             if self.has_boundary_conditions_derivative:
+
+                endPoint = test_solution.mesh.size - 1
+
                 if self.num_parameters > 0:
-    
-                    bc_derivative_calc, bc_param_derivative_calc = self.boundary_conditions_derivativep(test_solution.mesh[0], test_solution.solution[:, 0], test_solution.parameters)
-                    func_derivative_num = numpy.zeros((self.num_ODE,self.num_ODE))
-                    func_param_derivative_num = numpy.zeros( (self.num_ODE, self.num_parameters))
-    
+
+                    bcA_derivative_calc,bcB_derivative_calc, bcA_param_derivative_calc, bcB_param_derivative_calc = self.boundary_conditions_derivativep(test_solution.solution[:, 0],
+                                                                                                                                                        test_solution.solution[:,endPoint],
+                                                                                                                                                        test_solution.parameters)
+
+                    bcA_derivative_num = numpy.zeros((self.num_left_boundary_conditions, self.num_ODE))
+                    bcB_derivative_num = numpy.zeros((self.num_ODE + self.num_parameters - self.num_left_boundary_conditions, self.num_ODE))
+                    bcA_param_derivative_num = numpy.zeros((self.num_left_boundary_conditions, self.num_parameters))
+                    bcB_param_derivative_num = numpy.zeros( (self.num_ODE + self.num_parameters - self.num_left_boundary_conditions, self.num_parameters))
+
+                    # calculate analytic derivatives for
                     for i in range(self.num_ODE):
                         delta = numpy.zeros(self.num_ODE)
                         delta[i] += step
-    
-                        point1 = self.functionp(test_solution.mesh[1], test_solution.solution[:, 1], test_solution.parameters)
-                        point2 = self.functionp(test_solution.mesh[1], test_solution.solution[:, 1] + delta, test_solution.parameters)
-    
-                        func_derivative_num[:, i] = (point1 - point2)/step
-    
+
+                        point1A, point1B = self.boundary_conditionsp(test_solution.solution[:, 0],test_solution.solution[:, endPoint], test_solution.parameters)
+                        point2A, point2B = self.boundary_conditionsp(test_solution.solution[:, 0] + delta ,test_solution.solution[:, endPoint] + delta, test_solution.parameters)
+
+                        bcA_derivative_num[:, i] = (point2A - point1A)/step
+                        bcB_derivative_num[:, i] = (point2B - point1B)/step
+
                     for i in range(self.num_parameters):
                         delta = numpy.zeros(self.num_parameters)
                         delta[i] += step
-    
-                        point1 = self.functionp(test_solution.mesh[1], test_solution.solution[:, 1], test_solution.parameters)
-                        point2 = self.functionp(test_solution.mesh[1], test_solution.solution[:, 1], test_solution.parameters + delta)
-    
-                        func_param_derivative_num[:, i] = (point2 - point1)/step
-    
+
+                        point1A, point1B = self.boundary_conditionsp(test_solution.solution[:, 0],test_solution.solution[:, endPoint], test_solution.parameters)
+                        point2A, point2B = self.boundary_conditionsp(test_solution.solution[:, 0] ,test_solution.solution[:, endPoint], test_solution.parameters + delta)
+
+                        bcA_param_derivative_num[:, i] = (point2A - point1A)/step
+                        bcB_param_derivative_num[:, i] = (point2B - point1B)/step
+
                     # now compare the actual derivatives with the calculated ones
-    
-                    if not (numpy.round(func_derivative_calc - func_derivative_num, places) == 1).all():
-                        raise ValueError("analytical derivative matrix does not match numerical derivative matrix. Difference is \n" + str(func_derivative_calc - func_derivative_num))
-    
-                    if not (numpy.round(func_param_derivative_calc - func_param_derivative_num, places) == 0).all():
-                        raise ValueError("analytical derivative (with respect to parameters) matrix does not match numerical derivative matrix. Difference is \n" + str(func_param_derivative_calc - func_param_derivative_num))
+
+                    if not  (numpy.round(bcA_derivative_calc - bcA_derivative_num, places) == 0).all():
+                        raise ValueError("analytical derivative matrix for the left boundary condition does not match numerical derivative matrix.\n Analytical is:\n" + str(bcA_derivative_calc)
+                                     + "\nNumerical is:\n" + str(bcA_derivative_num))
+
+                    if not  (numpy.round(bcB_derivative_calc - bcB_derivative_num, places) == 0).all():
+                        raise ValueError("analytical derivative matrix for the right boundary condition does not match numerical derivative matrix.\n Analytical is:\n" + str(bcB_derivative_calc)
+                                     + "\nNumerical is:\n" + str(bcB_derivative_num))
+
+                    if not  (numpy.round(bcA_param_derivative_calc - bcA_param_derivative_num, places) == 0).all():
+                        raise ValueError("analytical derivative matrix (with respect to parameters) for the left boundary condition does not match numerical derivative matrix.\n Analytical is:\n" + str(bcA_param_derivative_calc)
+                                     + "\nNumerical is:\n" + str(bcA_param_derivative_num))
+
+                    if not  (numpy.round(bcB_param_derivative_calc - bcB_param_derivative_num, places) == 0).all():
+                        raise ValueError("analytical derivative matrix (with respect to parameters) for the left boundary condition does not match numerical derivative matrix.\n Analytical is:\n" + str(bcB_param_derivative_calc)
+                                     + "\nNumerical is:\n" + str(bcB_param_derivative_num))
+
                 else:
-                    func_derivative_calc = self.function_derivative(test_solution.mesh[0], test_solution.solution[:, 0])
-                    func_derivative_num = numpy.zeros((self.num_ODE,self.num_ODE))
-    
+                    bcA_derivative_calc,bcB_derivative_calc = self.boundary_conditions_derivativep(test_solution.solution[:, 0],
+                                                                                                   test_solution.solution[:,endPoint])
+
+                    bcA_derivative_num = numpy.zeros((self.num_left_boundary_conditions, self.num_ODE))
+                    bcB_derivative_num = numpy.zeros((self.num_ODE - self.num_left_boundary_conditions, self.num_ODE))
+
+                    # calculate analytic derivatives for
                     for i in range(self.num_ODE):
                         delta = numpy.zeros(self.num_ODE)
                         delta[i] += step
-    
-                        point1 = self.function(test_solution.mesh[1], test_solution.solution[:, 1])
-                        point2 = self.function(test_solution.mesh[1], test_solution.solution[:, 1] + delta)
-    
-                        func_derivative_num[:, i] = (point2 - point1)/step
-    
-    
-                    # now compare the actual derivatives with the calculated ones
-                    if not (numpy.round(func_derivative_calc - func_derivative_num, places) == 0).all():
-                        raise ValueError("analytical derivative matrix does not match numerical derivative matrix. Difference is \n" + str(func_derivative_calc - func_derivative_num))
 
+                        point1A, point1B = self.boundary_conditionsp(test_solution.solution[:, 0],test_solution.solution[:, endPoint])
+                        point2A, point2B = self.boundary_conditionsp(test_solution.solution[:, 0] + delta ,test_solution.solution[:, endPoint] + delta)
+
+                        bcA_derivative_num[:, i] = (point2A - point1A)/step
+                        bcB_derivative_num[:, i] = (point2B - point1B)/step
+
+                    # now compare the actual derivatives with the calculated ones
+
+
+                    if not  (numpy.round(bcA_derivative_calc - bcA_derivative_num, places) == 0).all():
+                        raise ValueError("analytical derivative matrix for the left boundary condition does not match numerical derivative matrix.\n Analytical is:\n" + str(bcA_derivative_calc)
+                                     + "\nNumerical is:\n" + str(bcA_derivative_num))
+
+                    if not  (numpy.round(bcB_derivative_calc - bcB_derivative_num, places) == 0).all():
+                        raise ValueError("analytical derivative matrix for the right boundary condition does not match numerical derivative matrix.\n Analytical is:\n" + str(bcB_derivative_calc)
+                                     + "\nNumerical is:\n" + str(bcB_derivative_num))
 
     @staticmethod
     def load(path):
